@@ -12,7 +12,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "System.hpp"
@@ -60,14 +60,14 @@ namespace afnix {
 
   Debugger::Debugger (void) {
     // initialize the debugger mode
-    d_emcs  = false;
-    d_strt  = false;
-    d_next   = false;
+    d_emcs = false;
+    d_strt = false;
+    d_next = false;
     d_fini = false;
-    d_exit   = false;
-    d_vflg  = true;
-    d_lmax   = 10;
-    p_form   = nullptr;
+    d_exit = false;
+    d_vflg = true;
+    d_lmax = 10;
+    p_form = nullptr;
     // initialize the terminal
     if (p_term != nullptr) {
       p_term->setpp   (DEFAULT_PROMPT1);
@@ -88,14 +88,14 @@ namespace afnix {
 
   Debugger::Debugger (const Debugger& that) : Interp (that) {
     // copy debugger mode
-    d_emcs  = that.d_emcs;
-    d_strt  = that.d_strt;
-    d_next   = that.d_next;
+    d_emcs = that.d_emcs;
+    d_strt = that.d_strt;
+    d_next = that.d_next;
     d_fini = that.d_fini;
-    d_exit   = that.d_exit;
-    d_vflg  = that.d_vflg;
-    d_lmax   = that.d_lmax;
-    d_fnam  = that.d_fnam;
+    d_exit = that.d_exit;
+    d_vflg = that.d_vflg;
+    d_lmax = that.d_lmax;
+    d_path = that.d_path;
     d_brkp = that.d_brkp;
     // copy debugger object
     Object::iref (p_dbrd = that.p_dbrd);
@@ -181,7 +181,7 @@ namespace afnix {
   // load a file name - first read the file - create a block form list
   // also updates the debugger tables - and finally execute it
 
-  void Debugger::load (const String& name) {
+  bool Debugger::load (const String& name, const bool lflg) {
     wrlock ();
     try {
       // try to find the instrumented module
@@ -191,9 +191,24 @@ namespace afnix {
       } else {
 	imod = dynamic_cast <Ifile*> (p_iset->find (name));
       }
+      // collect the module name and mark
+      if (imod == nullptr) {
+	unlock ();
+	return false;
+      }
+      String name = imod->getname ();
+      if (name.isnil () == false) {
+	if ((d_mtag.exists(name) == true) && (lflg == false)) {
+	  unlock ();
+	  return true;
+	}
+	// mark as load
+	d_mtag.add (name);
+      }
       // evaluate the ifile
       if (imod != nullptr) Object::cref (imod->eval (this, getgset ()));
       unlock ();
+      return true;
     } catch (...) {
       unlock ();
       throw;
@@ -295,11 +310,11 @@ namespace afnix {
 
   // set the initial file name
 
-  void Debugger::setinitial (const String& fname) {
+  void Debugger::setinitial (const String& path) {
     wrlock ();
     try {
-      if (fname.length () != 0) {
-	Ifile* ifile = instrument (fname);
+      if (path.isnil () == false) {
+	Ifile* ifile = instrument (path);
 	if ((ifile != nullptr) && (ifile->length () > 0)) {
 	  Iform* form = ifile->getform(0);
 	  if (form != nullptr) {
@@ -311,7 +326,7 @@ namespace afnix {
 	    info (name, lnum);
 	  }
 	}
-	d_fnam = fname;
+	d_path = path;
       }
       unlock ();
     } catch (...) {
@@ -325,11 +340,11 @@ namespace afnix {
   void Debugger::runinitial (void) {
     wrlock ();
     try {
-      if (d_fnam.length () == 0) {
+      if (d_path.length () == 0) {
 	throw Exception ("axd-error", "no initial file specified");
       }
       d_strt = true;
-      load (d_fnam);
+      load (d_path, false);
       unlock ();
     } catch (...) {
       unlock ();
@@ -339,11 +354,11 @@ namespace afnix {
 
   // instrument a file and return an instrumented file object
 
-  Ifile* Debugger::instrument (const String& fname) {
+  Ifile* Debugger::instrument (const String& path) {
     wrlock ();
     try {
       // bind the input stream
-      InputStream* ms = p_rslv->tolookup  (fname);
+      InputStream* ms = p_rslv->tolookup  (path);
       // get the interpreter encoding/transcoding modes
       if (ms != nullptr) {
 	// check for forced encoding
@@ -360,7 +375,7 @@ namespace afnix {
 	}
       }
       // try to open the module
-      String mn = p_rslv->tornam (fname);
+      String mn = p_rslv->tornam (path);
       Module mp (ms, mn);
       // bind the instrumented file
       Ifile* ifile = new Ifile (mp);
@@ -561,7 +576,7 @@ namespace afnix {
     wrlock ();
     try {
       OutputStream* os = getos ();
-      String fname = (d_fnam.length () == 0) ? "not defined" : d_fnam;
+      String fname = (d_path.length () == 0) ? "not defined" : d_path;
       String vflag = d_vflg ? "true" : "false";
       long   blen  = d_brkp.length ();
       String sform = (p_form == nullptr) ? "not defined" : p_form->getname ();
