@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #ifndef AFNIX_DEBUGGER_HPP
@@ -62,8 +62,8 @@ namespace afnix {
     bool d_vflg;
     /// the max number of lines
     long d_lmax;
-    /// the initial file name
-    String  d_fnam;
+    /// the initial path
+    String  d_path;
     /// the breakpoint vector
     Vector  d_brkp;
     /// the debugger reader
@@ -99,8 +99,10 @@ namespace afnix {
     /// @param nset the nameset context
     virtual bool loop (Nameset* nset) ;
 
-    /// load a file by doing a read-eval loop
-    void load (const String& name) override;
+    /// load a module ny name
+    /// @param name the name to resolve
+    /// @param lflg the load flag
+    bool load (const String& name, const bool lflg) override;
 
     /// break the evaluable in a nameset with an object
     /// @param nset   the nameset to loop
@@ -124,16 +126,16 @@ namespace afnix {
     /// set the exit flag
     virtual void setexit (const bool flag);
 
-    /// set the initial file name
-    /// @param fname the file name to set
-    virtual void setinitial (const String& fname);
+    /// set the initial file path
+    /// @param path the file path to set
+    virtual void setinitial (const String& path);
 
     /// run the initial file name
     virtual void runinitial (void);
 
-    /// instrument a file
-    /// @param fname the file name to instrument
-    virtual Ifile* instrument (const String& fname);
+    /// instrument a file by path
+    /// @param path the file path to instrument
+    virtual Ifile* instrument (const String& path);
 
     /// install a breakpoint at the current position
     virtual void setbpoint (void);
