@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "Ifile.hpp"
@@ -58,7 +58,7 @@ namespace afnix {
   // create a new ifile by name with a module
   
   Ifile::Ifile (Module& mp) {
-    d_fname = mp.getname ();
+    d_name = mp.getname ();
     while (true) {
       Form* form = mp.parse ();
       if (form == nullptr) break;
@@ -81,11 +81,11 @@ namespace afnix {
     return "Ifile";
   }
 
-  // return the i-file file name
+  // return the module name
 
-  String Ifile::getfname (void) const {
+  String Ifile::getname (void) const {
     rdlock ();
-    String result = d_fname;
+    String result = d_name;
     unlock ();
     return result;
   }
