@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #ifndef  AFNIX_BUILTIN_HPP
@@ -49,6 +49,7 @@ namespace afnix {
     static Object* sfforce    (Evaluable* zobj, Nameset* nset, Cons* args);
     static Object* sfdelay    (Evaluable* zobj, Nameset* nset, Cons* args);
     static Object* sfbreak    (Evaluable* zobj, Nameset* nset, Cons* args);
+    static Object* sfapply    (Evaluable* zobj, Nameset* nset, Cons* args);
     static Object* sffuture   (Evaluable* zobj, Nameset* nset, Cons* args);
     static Object* sflaunch   (Evaluable* zobj, Nameset* nset, Cons* args);
     static Object* sflambda   (Evaluable* zobj, Nameset* nset, Cons* args);
@@ -103,6 +104,7 @@ namespace afnix {
     static Object* sfprmp     (Evaluable* zobj, Nameset* nset, Cons* args);
     static Object* sfthrp     (Evaluable* zobj, Nameset* nset, Cons* args);
     static Object* sflogp     (Evaluable* zobj, Nameset* nset, Cons* args);
+    static Object* sfmodp     (Evaluable* zobj, Nameset* nset, Cons* args);
     static Object* sfbbufp    (Evaluable* zobj, Nameset* nset, Cons* args);
     static Object* sfhashp    (Evaluable* zobj, Nameset* nset, Cons* args);
     static Object* sffifop    (Evaluable* zobj, Nameset* nset, Cons* args);
@@ -133,6 +135,7 @@ namespace afnix {
     static Object* sfloadp    (Evaluable* zobj, Nameset* nset, Cons* args);
     static Object* sfrslvp    (Evaluable* zobj, Nameset* nset, Cons* args);
     static Object* sfviewp    (Evaluable* zobj, Nameset* nset, Cons* args);
+    static Object* sfloglp    (Evaluable* zobj, Nameset* nset, Cons* args);
     static Object* sfstrcp    (Evaluable* zobj, Nameset* nset, Cons* args);
     static Object* sflocalp   (Evaluable* zobj, Nameset* nset, Cons* args);
     static Object* sfregexp   (Evaluable* zobj, Nameset* nset, Cons* args);
@@ -144,6 +147,7 @@ namespace afnix {
     static Object* sfreadrp   (Evaluable* zobj, Nameset* nset, Cons* args);
     static Object* sflockfp   (Evaluable* zobj, Nameset* nset, Cons* args);
     static Object* sffuturp   (Evaluable* zobj, Nameset* nset, Cons* args);
+    static Object* sfhyperp   (Evaluable* zobj, Nameset* nset, Cons* args);
   };
 }
 
