@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "Set.hpp"
@@ -32,8 +32,8 @@
 #include "Engsrl.hxx"
 #include "Common.hpp"
 #include "Locale.hpp"
-#include "Logger.hpp"
 #include "Interp.hpp"
+#include "Hyperp.hpp"
 #include "Thrset.hpp"
 #include "Reader.hpp"
 #include "Module.hpp"
@@ -53,6 +53,7 @@
 #include "Closure.hpp"
 #include "Counter.hpp"
 #include "Rupture.hpp"
+#include "Loggable.hpp"
 #include "Function.hpp"
 #include "Instance.hpp"
 #include "Structure.hpp"
@@ -110,6 +111,7 @@ namespace afnix {
     interp->mkrsv ("force",          new Function (Builtin::sfforce));
     interp->mkrsv ("delay",          new Function (Builtin::sfdelay));
     interp->mkrsv ("break",          new Function (Builtin::sfbreak));
+    interp->mkrsv ("apply",          new Function (Builtin::sfapply));
     interp->mkrsv ("future",         new Function (Builtin::sffuture));
     interp->mkrsv ("lambda",         new Function (Builtin::sflambda));
     interp->mkrsv ("switch",         new Function (Builtin::sfswitch));
@@ -172,6 +174,7 @@ namespace afnix {
     interp->mkrsv ("buffer-p",       new Function (Builtin::sfbufp));
     interp->mkrsv ("thread-p",       new Function (Builtin::sfthrp));
     interp->mkrsv ("thrset-p",       new Function (Builtin::sfthrsp));
+    interp->mkrsv ("hyperp-p",       new Function (Builtin::sfhyperp));
     interp->mkrsv ("strvec-p",       new Function (Builtin::sfstvp));
     interp->mkrsv ("vector-p",       new Function (Builtin::sfvecp));
     interp->mkrsv ("bitset-p",       new Function (Builtin::sfbitsp));
@@ -182,6 +185,7 @@ namespace afnix {
     interp->mkrsv ("logger-p",       new Function (Builtin::sflogp));
     interp->mkrsv ("former-p",       new Function (Builtin::sfformrp));
     interp->mkrsv ("reader-p",       new Function (Builtin::sfreadrp));
+    interp->mkrsv ("module-p",       new Function (Builtin::sfmodp));
     interp->mkrsv ("future-p",       new Function (Builtin::sffuturp));
     interp->mkrsv ("lexical-p",      new Function (Builtin::sflexp));
     interp->mkrsv ("condvar-p",      new Function (Builtin::sfcondp));
@@ -199,6 +203,7 @@ namespace afnix {
     interp->mkrsv ("constant-p",     new Function (Builtin::sfcstp));
     interp->mkrsv ("nameable-p",     new Function (Builtin::sfnblp));
     interp->mkrsv ("viewable-p",     new Function (Builtin::sfviewp));
+    interp->mkrsv ("loggable-p",     new Function (Builtin::sfloglp));
     interp->mkrsv ("instance-p",     new Function (Builtin::sfinstp));
     interp->mkrsv ("resolver-p",     new Function (Builtin::sfrslvp));
     interp->mkrsv ("property-p",     new Function (Builtin::sfpropp));
@@ -234,12 +239,14 @@ namespace afnix {
     interp->mkrsv ("Vector",         new Meta (Vector::mknew)); 
     interp->mkrsv ("Thread",         new Meta (Thread::mknew));
     interp->mkrsv ("Thrset",         new Meta (Thrset::mknew));
+    interp->mkrsv ("Hyperp",         new Meta (Hyperp::mknew));
     interp->mkrsv ("Bitset",         new Meta (Bitset::mknew));
     interp->mkrsv ("Symbol",         new Meta (Symbol::mknew));
     interp->mkrsv ("Locale",         new Meta (Locale::mknew));
     interp->mkrsv ("Logger",         new Meta (Logger::meval, Logger::mknew));
     interp->mkrsv ("Lexer",          new Meta (Lexer::mknew));
     interp->mkrsv ("Reader",         new Meta (Reader::mknew));
+    interp->mkrsv ("Module",         new Meta (Module::mknew));
     interp->mkrsv ("Lexical",        new Meta (Lexical::mknew));
     interp->mkrsv ("Condvar",        new Meta (Condvar::mknew));    
     interp->mkrsv ("Relatif",        new Meta (Relatif::mknew));    
@@ -250,6 +257,7 @@ namespace afnix {
     interp->mkrsv ("Closure",        new Meta (Closure::mknew));
     interp->mkrsv ("Counter",        new Meta (Counter::mknew));
     interp->mkrsv ("Message",        new Meta (Message::meval, Message::mknew));
+    interp->mkrsv ("Loggable",       new Meta (Loggable::mknew));
     interp->mkrsv ("Instance",       new Meta (Instance::mknew));
     interp->mkrsv ("Resolver",       new Meta (Resolver::mknew));
     interp->mkrsv ("Property",       new Meta (Property::mknew));
@@ -278,6 +286,7 @@ namespace afnix {
   static const String F_NOP_MSG = "    [f nopath]\t do not set initial path";
   static const String F_SRE_MSG = "    [f   seed]\t seed random engine";
   static const String F_NOS_MSG = "    [f noseed]\t do not seed random engine";
+  static const String F_STC_MSG = "    [f strict]\t set strict mode";
 
   // get an interpreter option class
 
@@ -285,11 +294,12 @@ namespace afnix {
     // create the option class
     Options* opts = new Options (U_CLS_MSG);
 
-    // add the list options
+    // add the standard options
     opts->add ('f', "seed",   F_SRE_MSG);
     opts->add ('f', "noseed", F_NOS_MSG);
     opts->add ('f', "nopath", F_NOP_MSG);
     opts->add ('f', "assert", F_ASR_MSG);
+    opts->add ('f', "strict", F_STC_MSG);
 
     // add the string options
     opts->add (Options::SOPT, 'e', E_OPT_MSG);
@@ -315,6 +325,7 @@ namespace afnix {
   Interp::Interp (void) {
     // initialize default values
     d_clon = false;
+    d_smod = SMOD_NONE;
     // create a default terminal
     Object::iref (p_term = new Terminal);
     // save streams
@@ -340,6 +351,7 @@ namespace afnix {
   Interp::Interp (const bool tflg) {
     // initialize default values
     d_clon = false;
+    d_smod = SMOD_NONE;
     // set the streams
     if (tflg == true) {
       Object::iref (p_term = new Terminal);
@@ -371,6 +383,7 @@ namespace afnix {
   Interp::Interp (InputStream* is, OutputStream* os, OutputStream* es) {
     // initialize default values
     d_clon = false;
+    d_smod = SMOD_NONE;
     p_term = nullptr;
     // save streams
     Object::iref (p_is = is);
@@ -390,11 +403,12 @@ namespace afnix {
     Object::iref (p_shld = new Loader);
   }
 
-  // copy constructor
+  // copy construct this interpreter
 
   Interp::Interp (const Interp& that) {
     // initialize default value
     d_clon = true;
+    d_smod = that.d_smod;
     // copy the terminal
     Object::iref (p_term = that.p_term);
     // copy the streams
@@ -415,7 +429,7 @@ namespace afnix {
     Object::iref (p_shld = that.p_shld);
   }
 
-  // delete this interpreter
+  // destroy this interpreter
 
   Interp::~Interp (void) {
     // protect us
@@ -572,7 +586,7 @@ namespace afnix {
     // load the file
     bool status = true;
     try {
-      load (name);
+      load (name, false);
     } catch (const Exception& e) {
       status = false;
       getes()->errorln (e);
@@ -586,19 +600,24 @@ namespace afnix {
     return status;
   }
 
-  // loop with a module name by doing a read-eval loop - no thread wait
+  // load an object in the interpreter
   
-  void Interp::load (const String& name) {
-    load (name, name, false);
+  bool Interp::load (Object* lobj, const bool lflg) {
+    // check for a string
+    auto sobj = dynamic_cast<String*>(lobj);
+    if (sobj != nullptr) return load (*sobj, lflg);
+    // check for a module
+    auto mobj = dynamic_cast<Module*>(lobj);
+    if (mobj != nullptr) return load (*mobj, lflg);
+    // invalid object
+    return false;
   }
   
-  // loop with a module name by doing a read-eval loop - no thread wait
+  // load a module in the interpreter
   
-  void Interp::load (const String& name, const String& mtag, const bool fflg) {
+ bool Interp::load (const String& name, const bool lflg) {
     // check for valid name and tag
-    if ((name.isnil() || mtag.isnil ()) == true) return;
-    // check if the module is already loaded
-    if ((d_mtag.exists (mtag) == true) && (fflg == false)) return;
+    if (name.isnil () == true) return false;
     // bind the input stream
     t_transient<InputStream> ms = p_rslv->tolookup  (name);
     // get the interpreter encoding/transcoding modes
@@ -619,9 +638,22 @@ namespace afnix {
     // try to open the module
     String mn = p_rslv->tornam (name);
     Module mp (ms.detach (), mn);
-    // mark the module as loaded
-    d_mtag.add (mtag);
-    // loop until we have an eos
+    // load the module
+    return load (mp, lflg);
+  }
+
+  // load a module in the interpreter
+
+  bool Interp::load (Module& mp, const bool lflg) {
+    // get the module name
+    String name = mp.getname ();
+    // check and mark as loaded
+    if (name.isnil () == false) {
+      if ((d_mtag.exists(name) == true) && (lflg == false)) return true;
+      // mark the module as loaded
+      d_mtag.add (name);
+    }
+    // loop until we has a null form
     while (true) {
       Form* form = nullptr;
       try {
@@ -646,6 +678,7 @@ namespace afnix {
 	throw;
       }
     }
+    return true;
   }
 
   // return the interpreter input stream
@@ -1011,9 +1044,8 @@ namespace afnix {
       Strvec result;
       long len = p_argv->length ();
       for (long i = 0; i < len; i++) {
-	String* str = dynamic_cast <String*> (p_argv->get (i));
-	if (str == nullptr) continue;
-	result.add (*str);
+	auto lobj = dynamic_cast <Literal*> (p_argv->get (i));
+	if (lobj != nullptr) result.add (lobj->tostring());
       }
       unlock ();
       return result;
@@ -1023,6 +1055,20 @@ namespace afnix {
     }
   }
 
+  // get the interpreter argument vector
+
+  Vector* Interp::getargv (void) const {
+    rdlock ();
+    try {
+      Vector* result = p_argv;
+      unlock ();
+      return result;
+    } catch (...) {
+      unlock ();
+      throw;
+    }
+  }
+  
   // add a new path entry in the resolver
 
   void Interp::addpath (const String& path) {
@@ -1374,9 +1420,8 @@ namespace afnix {
 	return nullptr;
       }
       if (quark == QUARK_LOAD) {
-	String name = argv->getstring (0);
-	load (name);
-	return nullptr;
+	Object* lobj = argv->get (0);
+	return new Boolean (load (lobj, false));
       }
       if (quark == QUARK_DUP) {
 	Object*    obj = argv->get (0);
@@ -1438,10 +1483,9 @@ namespace afnix {
 	return dup (is, os, os);
       }
       if (quark == QUARK_LOAD) {
-	String name = argv->getstring (0);
-	bool   fflg = argv->getbool (1);
-	load (name, name, fflg);
-	return nullptr;
+	Object* lobj = argv->get (0);
+	bool    lflg = argv->getbool (1);
+	return new Boolean (load (lobj, lflg));
       }
     }
     // check for 3 arguments
@@ -1470,13 +1514,6 @@ namespace afnix {
 	}
 	return dup (is, os, es);
       }
-      if (quark == QUARK_LOAD) {
-	String name = argv->getstring (0);
-	String mtag = argv->getstring (1);
-	bool   fflg = argv->getbool (2);
-	load (name, mtag, fflg);
-	return nullptr;
-      }
     }
     // call the runnable method
     return Runnable::apply (zobj, nset, quark, argv);
