@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #ifndef  AFNIX_INTERP_HPP
@@ -21,6 +21,10 @@
 #include "Loader.hpp"
 #endif
 
+#ifndef  AFNIX_MODULE_HPP
+#include "Module.hpp"
+#endif
+
 #ifndef  AFNIX_OPTIONS_HPP
 #include "Options.hpp"
 #endif
@@ -61,6 +65,12 @@ namespace afnix {
 
   class Interp : public virtual Runnable {
   public:
+    /// the security mode
+    enum t_smod {
+      SMOD_NONE, // open mode
+      SMOD_STRC  // strict mode
+    };
+    
     /// @return an interpreter option object
     static Options* getopts (void);
     
@@ -71,6 +81,8 @@ namespace afnix {
     mutable OutputStream* p_os;
     /// the standard error stream
     mutable OutputStream* p_es;
+    /// the scurity mode
+    t_smod  d_smod;
     /// the cloned interpreter flag
     bool    d_clon;
     /// the posted object
@@ -145,17 +157,20 @@ namespace afnix {
     /// @return false if something bad happen
     bool loop (const String& name) override;
 
-    /// run the read-eval loop with a module
-    /// @param name the module name to read
-    /// @param mtag the module tag
-    /// @param fflg the force flag
-    void load (const String& name) override;
-    
-    /// run the read-eval loop with a module
-    /// @param name the module name to read
-    /// @param mtag the module tag
-    /// @param fflg the force flag
-    virtual void load (const String& name, const String& mtag, const bool fflg);
+    /// load an object in the interpter
+    /// @param lobj the object to load
+    /// @param lflg the load flag
+    bool load (Object* lobj, const bool lflg) override;
+
+    /// load a module in the interpreter
+    /// @param name the module name
+    /// @param mflg the load flag
+    virtual bool load (const String& name, const bool lflg);
+
+    /// load a module in the interpreter
+    /// @param mp   the module object to load
+    /// @param mflg the load flag
+    virtual bool load (Module& mp, const bool lflg);
 
     /// @return the interpreter input stream
     InputStream* getis (void) const override;
@@ -236,6 +251,9 @@ namespace afnix {
     /// @return the interpreter arguments
     virtual Strvec getargs (void) const;
 
+    /// @return the interpreter argument vector
+    virtual Vector* getargv (void) const;
+    
     /// add a path to the resolver
     /// @param path the path to add
     virtual void addpath (const String& path);
