@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #ifndef  AFNIX_MODULE_HPP
@@ -21,8 +21,8 @@
 #include "Former.hpp"
 #endif
 
-#ifndef  AFNIX_INPUTFILE_HPP
-#include "InputFile.hpp"
+#ifndef  AFNIX_NAMEABLE_HPP
+#include "Nameable.hpp"
 #endif
 
 #ifndef  AFNIX_OUTPUTSTREAM_HPP
@@ -41,17 +41,20 @@ namespace afnix {
   class Module : public Nameable {
   public:
     /// supported module format
-    enum t_mtype {REGULAR, COMPILED};
+    enum t_mfmt
+      {
+	MFMT_NONE,
+	MFMT_TEXT,
+	MFMT_CBIN
+      };
 
   private:
-    /// the module type
-    t_mtype d_type;
+    /// the module format
+    t_mfmt d_mfmt;
     /// the module name
     String  d_name;
-    /// the input stream
-    InputStream* p_is;
     /// the form reader
-    Former* p_former;
+    Former* p_frmr;
 
   public:
     /// create an empty module
@@ -61,40 +64,77 @@ namespace afnix {
     /// @param name the module name
     Module (const String& name);
 
-    /// create a module with a stream
+    /// create a module by buffer
+    /// @param ibuf the input buffer
+    Module (const Buffer& ibuf);
+
+    /// create a module by stream and name
     /// @param is   the input stream to use
     /// @param name the optional stream name
     Module (InputStream* is, const String& name);
 
+    /// create a module by buffer and name
+    /// @param ibuf the input buffer
+    /// @param name the buffer name
+    Module (const Buffer& ibuf, const String& name);
+
     /// destroy this module
     ~Module (void);
 
     /// @return the class name
-    String repr (void) const;
+    String repr (void) const override;
 
+    /// bind a module by input stream
+    /// @param is the input stream to bind
+    virtual bool bind (InputStream* is);
+    
+    /// bind a module by input stream and name
+    /// @param is the input stream to bind
+    /// @param name the input name
+    virtual bool bind (InputStream* is, const String& name);
+    
     /// @return the next form
-    Form* parse (void);
+    virtual Form* parse (void);
 
-    /// @return true is the module is regular
-    bool isregular (void) const;
+    /// @return true for a text module
+    virtual bool istext (void) const;
 
-    /// @return the module type
-    t_mtype gettype (void) const;
+    /// @return the module format
+    virtual t_mfmt getmfmt (void) const;
 
     /// @return the module name
-    String getname (void) const;
+    virtual String getname (void) const;
 
     /// @return the reader line number
-    long getlnum (void) const;
+    virtual long getlnum (void) const;
 
     /// write the module content
-    void write (OutputStream& os);
+    virtual bool write (OutputStream& os);
 
+    /// @return a module buffer content
+    virtual Buffer tobuffer (void);
+    
   private:
     // make the copy constructor private
-    Module (const Module&);
+    Module (const Module&) =delete;
     // make the assignment operator private
-    Module& operator = (const Module&);
+    Module& operator = (const Module&) =delete;
+
+  public:
+    /// create a new object in a generic way
+    /// @param argv the argument vector
+    static Object* mknew (Vector* argv);
+    
+    /// @return true if the given quark is defined
+    bool isquark (const long quark, const bool hflg) const override;
+    
+    /// apply this object with a set of arguments and a quark
+    /// @param zobj  the current evaluable
+    /// @param nset  the current nameset
+    /// @param quark the quark to apply these arguments
+    /// @param argv  the arguments to apply
+    Object* apply (Evaluable* zobj, Nameset* nset, const long quark,
+		   Vector* argv) override;
   };
 }
 
