@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "Set.hpp"
@@ -30,6 +30,7 @@
 #include "Lockf.hpp"
 #include "Buffer.hpp"
 #include "Combo.hpp"
+#include "Module.hpp"
 #include "Loader.hpp"
 #include "Thrset.hpp"
 #include "Reader.hpp"
@@ -37,9 +38,9 @@
 #include "Vector.hpp"
 #include "Bitset.hpp"
 #include "Locale.hpp"
-#include "Logger.hpp"
 #include "Symbol.hpp"
 #include "Future.hpp"
+#include "Hyperp.hpp"
 #include "Numeral.hpp"
 #include "Complex.hpp"
 #include "Message.hpp"
@@ -52,6 +53,7 @@
 #include "Promise.hpp"
 #include "Closure.hpp"
 #include "Counter.hpp"
+#include "Loggable.hpp"
 #include "Constant.hpp"
 #include "Resolver.hpp"
 #include "Instance.hpp"
@@ -428,6 +430,24 @@ namespace afnix {
     return new Boolean (result);
   }
 
+  // loglp: loggable predicate
+
+  Object* Builtin::sfloglp (Evaluable* zobj, Nameset* nset, Cons* args) {
+    Object* obj = get_obj (zobj, nset, args, "loggable-p");
+    bool result = (dynamic_cast <Loggable*> (obj) == nullptr) ? false : true;
+    Object::cref (obj);
+    return new Boolean (result);
+  }
+
+  // modp: module predicate
+
+  Object* Builtin::sfmodp (Evaluable* zobj, Nameset* nset, Cons* args) {
+    Object* obj = get_obj (zobj, nset, args, "module-p");
+    bool result = (dynamic_cast <Module*> (obj) == nullptr) ? false : true;
+    Object::cref (obj);
+    return new Boolean (result);
+  }
+
   // bbufp: block buffer predicate
 
   Object* Builtin::sfbbufp (Evaluable* zobj, Nameset* nset, Cons* args) {
@@ -688,4 +708,13 @@ namespace afnix {
     Object::cref (obj);
     return new Boolean (result);
   }
+
+  // future predicate
+  
+  Object* Builtin::sfhyperp (Evaluable* zobj, Nameset* nset, Cons* args) {
+    Object* obj = get_obj (zobj, nset, args, " hyperp-p");
+    bool result = (dynamic_cast <Hyperp*> (obj) == nullptr) ? false : true;
+    Object::cref (obj);
+    return new Boolean (result);
+  }
 }
