@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #ifndef  AFNIX_READER_HPP
@@ -63,26 +63,26 @@ namespace afnix {
     ~Reader (void);
 
     /// @return the class name
-    String repr (void) const;
+    String repr (void) const override;
 
     /// @return a form from this reader or nil in case of eos
-    Form* parse (void);
+    Form* parse (void) override;
 
     /// set the reader file name
     /// @param name the reader file name
     void setname (const String& name);
 
     /// @return the reader file name
-    String getname (void) const;
+    String getname (void) const override;
 
     /// @return an approximate line number for this reader
-    long getlnum (void) const;
+    long getlnum (void) const override;
 
   private:
     // make the copy constructor private
-    Reader (const Reader&);
+    Reader (const Reader&) =delete;
     // make the assignement operator private
-    Reader& operator = (const Reader&);
+    Reader& operator = (const Reader&) =delete;
     // private reader procedures
     Form* rform (bool);
     Form* bform (bool);
@@ -93,7 +93,7 @@ namespace afnix {
     static Object* mknew (Vector* argv);
 
     /// @return true if the given quark is defined
-    bool isquark (const long quark, const bool hflg) const;
+    bool isquark (const long quark, const bool hflg) const override;
 
     /// apply this object with a set of arguments and a quark
     /// @param zobj  the current evaluable
@@ -101,7 +101,7 @@ namespace afnix {
     /// @param quark the quark to apply these arguments
     /// @param argv  the arguments to apply
     Object* apply (Evaluable* zobj, Nameset* nset, const long quark,
-                   Vector* argv);
+                   Vector* argv) override;
 
   };
 }
