@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "Module.hpp"
@@ -23,9 +23,9 @@ int main (int, char**) {
   Module md;
   if (md.repr () != "Module") return 1;
 
-  // check for regular
-  if (md.isregular () == true) return 1;
-  if (md.gettype () != Module::COMPILED) return 1;
+  // check for none
+  if (md.istext () == true) return 1;
+  if (md.getmfmt () != Module::MFMT_NONE) return 1;
 
   // success
   return 0;
