@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "Vector.hpp"
@@ -321,10 +321,10 @@ namespace afnix {
     if (argc == 1) {
       Object* obj = argv->get (0);
       // check for a string
-      String* sobj = dynamic_cast <String*> (obj);
+      auto* sobj = dynamic_cast <String*> (obj);
       if (sobj != nullptr) return new InputMapped (*sobj);
       // check for a buffer
-      Buffer* bobj = dynamic_cast <Buffer*> (obj);
+      auto* bobj = dynamic_cast <Buffer*> (obj);
       if (bobj != nullptr) return new InputMapped (*bobj);
       // invalid object
       throw Exception ("type-error", "invalid object with mapped input strean",
