@@ -11,19 +11,19 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "Item.hpp"
 #include "Date.hpp"
 #include "Vector.hpp"
 #include "Logger.hpp"
+#include "Logger.hxx"
 #include "Common.hpp"
 #include "Utility.hpp"
 #include "Boolean.hpp"
 #include "Integer.hpp"
 #include "QuarkZone.hpp"
-#include "Exception.hpp"
 #include "OutputFile.hpp"
 
 namespace afnix {
@@ -360,14 +360,6 @@ namespace afnix {
   // - object section                                                        -
   // -------------------------------------------------------------------------
 
-  // the object eval quarks
-  static const long QUARK_LOGGER   = String::intern ("Logger");
-  static const long QUARK_MLVLFATL = String::intern ("FATAL");
-  static const long QUARK_MLVLXERR = String::intern ("ERROR");
-  static const long QUARK_MLVLWRNG = String::intern ("WARNING");
-  static const long QUARK_MLVLINFO = String::intern ("INFO");
-  static const long QUARK_MLVLDBUG = String::intern ("DEBUG");
-
   // the quark zone
   static const long QUARK_ZONE_LENGTH = 11;
   static QuarkZone  zone (QUARK_ZONE_LENGTH);
@@ -385,43 +377,6 @@ namespace afnix {
   static const long QUARK_GETMTIM = zone.intern ("get-message-time");
   static const long QUARK_GETMLVL = zone.intern ("get-message-level");
   
-  // map an item to a message level type
-  static inline Logger::t_mlvl item_to_mlvl (const Item& item) {
-    // check for a key item
-    if (item.gettid () != QUARK_LOGGER)
-      throw Exception ("item-error", "item is not a logger item");
-    // map the item to the enumeration
-    long quark = item.getquark ();
-    if (quark == QUARK_MLVLFATL) return Logger::MLVL_FATL;
-    if (quark == QUARK_MLVLXERR) return Logger::MLVL_XERR;
-    if (quark == QUARK_MLVLWRNG) return Logger::MLVL_WRNG;
-    if (quark == QUARK_MLVLINFO) return Logger::MLVL_INFO;
-    if (quark == QUARK_MLVLDBUG) return Logger::MLVL_DBUG;
-    throw Exception ("item-error", "cannot map item to message level type");
-  }
-  
-  // map a key type to an item
-  static inline Item* mlvl_to_item (const Logger::t_mlvl type) {
-    switch (type) {
-    case Logger::MLVL_FATL:
-      return new Item (QUARK_LOGGER, QUARK_MLVLFATL);
-      break;
-    case Logger::MLVL_XERR:
-      return new Item (QUARK_LOGGER, QUARK_MLVLXERR);
-      break;
-    case Logger::MLVL_WRNG:
-      return new Item (QUARK_LOGGER, QUARK_MLVLWRNG);
-      break;
-    case Logger::MLVL_INFO:
-      return new Item (QUARK_LOGGER, QUARK_MLVLINFO);
-      break;
-    case Logger::MLVL_DBUG:
-      return new Item (QUARK_LOGGER, QUARK_MLVLDBUG);
-      break;
-    }
-    return nullptr;
-  }
-  
   // evaluate an object data member
 
   Object* Logger::meval (Evaluable* zobj, Nameset* nset, const long quark) {
