@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "Math.hpp"
@@ -69,6 +69,12 @@ namespace afnix {
     return c_isinf (x);
   }
 
+  // check if a number is a real number
+
+  bool Math::isreal (const t_real x) {
+    return (!Math::isnan(x) && !Math::isinf (x));
+  }
+
   // compare two numbers by absolute precision
 
   bool Math::acmp (const t_real x, const t_real y) {
