@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #ifndef  AFNIX_MATH_HPP
@@ -64,6 +64,10 @@ namespace afnix {
     /// @param x the real argument
     static bool isinf (const t_real x);
 
+    /// check if the argument is a real
+    /// @param x the real argument
+    static bool isreal (const t_real x);
+
     /// compare two numbers by absolute precision
     /// @param x the reference argument
     /// @param y the argument to compare
