@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "Ascii.hpp"
@@ -690,7 +690,7 @@ namespace afnix {
     long ulen = Ascii::strlen (s);
     if (ulen == 0L) return nullptr;
     // decode as a utf string string
-    t_transient<t_quad> sbuf = ctoq_utf8 (s, ulen);
+    t_transient<t_quad,true> sbuf = ctoq_utf8 (s, ulen);
     // compute buffer length
     long blen = Unicode::strlen (*sbuf);
     // normalize the result
@@ -703,7 +703,7 @@ namespace afnix {
   t_quad* Unicode::strmak (const t_quad* s, const t_quad c) {
     // compute size
     long slen = Unicode::strlen (s);
-    t_transient<t_quad> sbuf = new t_quad[slen+2];
+    t_transient<t_quad,true> sbuf = new t_quad[slen+2];
     // copy string directly
     for (long k = 0L; k < slen; k++) sbuf[k] = s[k];
     sbuf[slen]   = c;
@@ -725,7 +725,7 @@ namespace afnix {
   t_quad* Unicode::strmak (const t_quad c, const t_quad* s) {
     // compute size
     long slen = Unicode::strlen (s);
-    t_transient<t_quad> sbuf = new t_quad[slen+2];
+    t_transient<t_quad,true> sbuf = new t_quad[slen+2];
     // add character and copy string
     sbuf[0] = c;
     for (long k = 0L; k < slen; k++) sbuf[k+1] = s[k];
@@ -742,7 +742,7 @@ namespace afnix {
     long len1 = Unicode::strlen (s1);
     long len2 = Ascii::strlen   (s2);
     // allocate a temporary buffer and copy
-    t_transient<t_quad> sbuf = new t_quad[len1+len2+1];
+    t_transient<t_quad,true> sbuf = new t_quad[len1+len2+1];
     for (long k = 0L; k < len1; k++) sbuf[k] = s1[k];
     for (long k = 0L; k < len2; k++) sbuf[len1+k] = Unicode::toquad (s2[k]);
     sbuf[len1+len2] = nilq;
@@ -758,7 +758,7 @@ namespace afnix {
     long len1 = Unicode::strlen (s1);
     long len2 = Unicode::strlen (s2);
     // allocate a temporary buffer and copy
-    t_transient<t_quad> sbuf = new t_quad[len1+len2+1];
+    t_transient<t_quad,true> sbuf = new t_quad[len1+len2+1];
     for (long k = 0L; k < len1; k++) sbuf[k]      = s1[k];
     for (long k = 0L; k < len2; k++) sbuf[len1+k] = s2[k];
     sbuf[len1+len2] = nilq;
@@ -785,7 +785,7 @@ namespace afnix {
     // get the buffer length and check for nil
     long slen = Ascii::strlen (s);
     // convert the source buffer
-    t_transient<t_quad> sbuf = new t_quad[slen+1];
+    t_transient<t_quad,true> sbuf = new t_quad[slen+1];
     for (long k = 0L; k < slen; k++) sbuf[k] = Unicode::toquad (s[k]);
     sbuf[slen] = nilq;
     // normalize if requested
@@ -799,7 +799,7 @@ namespace afnix {
     // get the string length and check for nil
     long slen = Unicode::strlen (s);
     // create a new quad array
-    t_transient<t_quad> sbuf = new t_quad[slen+1];
+    t_transient<t_quad,true> sbuf = new t_quad[slen+1];
     // copy the source buffer
     for (long k = 0L; k < slen; k++) sbuf[k] = s[k];
     sbuf[slen] = nilq;
@@ -811,7 +811,7 @@ namespace afnix {
 
   t_quad* Unicode::strdup (const char* s, const long size) {
     // create a new quad array
-    t_transient<t_quad> result = new t_quad[size+1];
+    t_transient<t_quad,true> result = new t_quad[size+1];
     // convert the source buffer
     for (long k = 0L; k < size; k++) result[k] = Unicode::toquad (s[k]);
     result[size] = nilq;
@@ -822,7 +822,7 @@ namespace afnix {
 
   t_quad* Unicode::strdup (const t_quad* s, const long size) {
     // create a new quad array
-    t_transient<t_quad> result = new t_quad[size+1];
+    t_transient<t_quad,true> result = new t_quad[size+1];
     // copy the source buffer
     for (long k = 0L; k < size; k++) result[k] = s[k];
     result[size] = nilq;
@@ -880,7 +880,7 @@ namespace afnix {
     // get the length and check
     long len = Ascii::strlen (s);
     if (len == 0) return c_ucdnil ();
-    t_transient<char> buf = Ascii::strdup (s);
+    t_transient<char,true> buf = Ascii::strdup (s);
     char* end = *buf + len - 1;
     // remove trailing blank
     while ((end != s) && ((*end == blkc) || (*end == tabc))) *end-- = nilc;
@@ -895,7 +895,7 @@ namespace afnix {
     // get the length and check
     long len = Ascii::strlen (s);
     if (len == 0) return c_ucdnil ();
-    t_transient<char> buf = Ascii::strdup (s);
+    t_transient<char,true> buf = Ascii::strdup (s);
     char* end = *buf + len - 1;
     // remove trailing blank
     while ((end != s) && (is_csep (*end, sep) == true)) *end-- = nilc;
@@ -910,7 +910,7 @@ namespace afnix {
     // get the length and check
     long len = Unicode::strlen (s);
     if (len == 0) return c_ucdnil ();
-    t_transient<t_quad> buf = Unicode::strdup (s);
+    t_transient<t_quad,true> buf = Unicode::strdup (s);
     t_quad* end = *buf + len - 1;
     // remove trailing blank
     while ((end != s) && ((*end == blkq) || (*end == tabq))) *end-- = nilq;
@@ -925,7 +925,7 @@ namespace afnix {
     // get the length and check
     long len = Unicode::strlen (s);
     if (len == 0) return c_ucdnil ();
-    t_transient<t_quad> buf = Unicode::strdup (s);
+    t_transient<t_quad,true> buf = Unicode::strdup (s);
     t_quad* end = *buf + len - 1;
     // remove trailing blank
     while ((end != s) && (is_qsep (*end, sep) == true)) *end-- = nilq;
@@ -984,7 +984,7 @@ namespace afnix {
     long len = Ascii::strlen (s);
     // allocate and convert
     long size = len * UCD_LCM_MAX + 1;
-    t_transient<t_quad> sbuf = new t_quad[size];
+    t_transient<t_quad,true> sbuf = new t_quad[size];
     long    sidx = 0;
     t_quad  sdst[UCD_LCM_MAX];
     for (long i = 0; i < len; i++) {
@@ -1006,7 +1006,7 @@ namespace afnix {
     long len = Unicode::strlen (s);
     // allocate and convert
     long size = len * UCD_LCM_MAX + 1;
-    t_transient<t_quad> sbuf = new t_quad[size];
+    t_transient<t_quad,true> sbuf = new t_quad[size];
     // set indexes
     long   sidx = 0;
     t_quad sdst[UCD_LCM_MAX];
@@ -1029,7 +1029,7 @@ namespace afnix {
     long len = Ascii::strlen (s);
     // allocate and convert
     long size = len * UCD_UCM_MAX + 1;
-    t_transient<t_quad> sbuf = new t_quad[size];
+    t_transient<t_quad,true> sbuf = new t_quad[size];
     // set indexes
     long    sidx = 0;
     t_quad  sdst[UCD_UCM_MAX];
@@ -1052,7 +1052,7 @@ namespace afnix {
     long len = Unicode::strlen (s);
     // allocate and convert
     long size = len * UCD_UCM_MAX + 1;
-    t_transient<t_quad> sbuf = new t_quad[size];
+    t_transient<t_quad,true> sbuf = new t_quad[size];
     // set indexes
     long    sidx = 0;
     t_quad  sdst[UCD_UCM_MAX];
@@ -1355,7 +1355,7 @@ namespace afnix {
   // encode a string based on a mode
 
   char* Unicode::encode (const Encoding::t_emod emod, const String& s) {
-    t_transient<t_quad> sbuf = s.toquad ();
+    t_transient<t_quad,true> sbuf = s.toquad ();
     char* cbuf = Unicode::encode (emod, *sbuf);
     return cbuf;
   }
