@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #ifndef  AFNIX_UTILITY_HPP
@@ -250,6 +250,9 @@ namespace afnix {
     /// @param hbuf the buffer to hash
     /// @param size the buffer size
     static long hashl (const t_byte* hbuf, const long size);
+
+    /// hash a string as a string
+    static String hashs (const String& sval);
   };
 }
 
