@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "Byte.hpp"
@@ -21,9 +21,9 @@
 #include "Vector.hpp"
 #include "Boolean.hpp"
 #include "Integer.hpp"
+#include "Character.hpp"
 #include "Evaluable.hpp"
 #include "QuarkZone.hpp"
-#include "Character.hpp"
 #include "Exception.hpp"
 #include "InputStream.hpp"
 #include "OutputStream.hpp"
@@ -571,12 +571,13 @@ namespace afnix {
 
   // merge a vector into this one
 
-  void Vector::merge (const Vector& v) {
+  Vector& Vector::merge (const Vector& v) {
     wrlock ();
     try {
       long len = v.length ();
       for (long i = 0; i < len; i++) add (v.get (i));
       unlock ();
+      return *this;
     } catch (...) {
       unlock ();
       throw;
