@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #ifndef  AFNIX_VECTOR_HPP
@@ -155,7 +155,7 @@ namespace afnix {
 
     /// merge a vector into this one
     /// @param v the vector to merge
-    virtual void merge (const Vector& v);
+    virtual Vector& merge (const Vector& v);
 
     /// @return a long integer value by object index
     virtual t_long getlong (const long index) const;
