@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "Math.hpp"
@@ -25,6 +25,11 @@ int main (int, char**) {
   // check for infinite
   if (Math::isinf (Math::CV_PINF) == false) return 1;
   if (Math::isinf (Math::CV_NINF) == false) return 1;
+  // check for real
+  if (Math::isreal (0.0) == false) return 1;
+  if (Math::isreal (Math::CV_NAN)  == true) return 1;
+  if (Math::isreal (Math::CV_PINF) == true) return 1;
+  if (Math::isreal (Math::CV_NINF) == true) return 1;
   
   // check for absolute precision
   if (Math::acmp (1.0, 1.0 + Math::d_aeps / 10.0) == false) return 1;
