@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "Meta.hpp"
@@ -20,6 +20,7 @@
 #include "Nwgsrl.hxx"
 #include "Libnwg.hpp"
 #include "Basexx.hpp"
+#include "Session.hpp"
 #include "Prednwg.hpp"
 #include "UriPath.hpp"
 #include "UriQuery.hpp"
@@ -52,6 +53,7 @@ namespace afnix {
     gset->symcst ("Cookie",             new Meta (Cookie::mknew));
     gset->symcst ("Basexx",             new Meta (Basexx::meval,
 						  Basexx::mknew));
+    gset->symcst ("Session",            new Meta (Session::mknew));
     gset->symcst ("UriPath",            new Meta (UriPath::mknew));
     gset->symcst ("UriQuery",           new Meta (UriQuery::mknew));
     gset->symcst ("Hyperlink",          new Meta (Hyperlink::mknew));
@@ -67,6 +69,7 @@ namespace afnix {
     gset->symcst ("json-p",             new Function (nwg_jsonp));
     gset->symcst ("cookie-p",           new Function (nwg_cookp));
     gset->symcst ("basexx-p",           new Function (nwg_bsxxp));
+    gset->symcst ("session-p",          new Function (nwg_sessp));
     gset->symcst ("uri-path-p",         new Function (nwg_uripp));
     gset->symcst ("uri-query-p",        new Function (nwg_uriqp));
     gset->symcst ("hyperlink-p",        new Function (nwg_hlnkp));
