@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #ifndef  AFNIX_PREDNWG_HPP
@@ -99,6 +99,12 @@ namespace afnix {
   /// @param args the arguments list
   Object* nwg_cjarp (Evaluable* zobj, Nameset* nset, Cons* args);
 
+  /// the session object predicate
+  /// @param zobj the current evaluable
+  /// @param nset the current nameset
+  /// @param args the arguments list
+  Object* nwg_sessp (Evaluable* zobj, Nameset* nset, Cons* args);
+
   /// the basexx codec object predicate
   /// @param zobj the current evaluable
   /// @param nset the current nameset
