@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------------
 // - Session.cpp                                                             -
-// - afnix:nwg module - http session class implementation                    -
+// - afnix:csm service - session class implementation                        -
 // ---------------------------------------------------------------------------
 // - This program is free software;  you can redistribute it  and/or  modify -
 // - it provided that this copyright notice is kept intact.                  -
@@ -11,15 +11,19 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2011 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "Time.hpp"
 #include "Nwgsid.hxx"
 #include "Vector.hpp"
 #include "System.hpp"
+#include "Utility.hpp"
 #include "Session.hpp"
 #include "Boolean.hpp"
+#include "Integer.hpp"
+#include "Relatif.hpp"
+#include "Evaluable.hpp"
 #include "QuarkZone.hpp"
 #include "Exception.hpp"
 #include "InputStream.hpp"
@@ -31,12 +35,14 @@ namespace afnix {
   // - private section                                                       -
   // -------------------------------------------------------------------------
 
-  // this procedure returns a new session for deserialization
-  static Serial* mksob (void) {
-    return new Session;
+  // the random session key size
+  static const long SESS_KEY_SIZE = 1024;
+
+  // generate a random session key
+  static String sess_new_skey (void) {
+    Relatif rn = Relatif::random (SESS_KEY_SIZE);
+    return rn.tohstr ();
   }
-  // register this session serial id
-  static const t_byte SERIAL_ID = Serial::setsid (SERIAL_SESS_ID, mksob);
 
   // -------------------------------------------------------------------------
   // - class section                                                         -
@@ -45,44 +51,89 @@ namespace afnix {
   // create an empty session
 
   Session::Session (void) {
+    d_skey = sess_new_skey ();
     d_path = System::tempnam ();
     d_ctim = Time::gettclk ();
     d_mtim = d_ctim;
-    d_etim = 0;
+    d_mage = 0LL;
   }
 
   // create a session by name
 
-  Session::Session (const String& name) {
-    d_name = name;
-    d_user = name;
+  Session::Session (const String& name) : Taggable (name) {
+    d_skey = sess_new_skey ();
     d_path = System::tempnam ();
     d_ctim = Time::gettclk ();
     d_mtim = d_ctim;
-    d_etim = 0;
+    d_mage = 0LL;
   }
 
-  // create a session by name and user
+  // create a session by name and info
 
-  Session::Session (const String& name, const String& user) {
-    d_name = name;
-    d_user = user;
+  Session::Session (const String& name, const String& info) :
+    Taggable (name, info) {
+    d_skey = sess_new_skey ();
     d_path = System::tempnam ();
     d_ctim = Time::gettclk ();
     d_mtim = d_ctim;
-    d_etim = 0;
+    d_mage = 0LL;
   }
 
-  // create a session by name, user and path
+  // create a session by name, info and maximum age
 
-  Session::Session (const String& name, const String& user, 
-		    const String& path) {
-    d_name = name;
-    d_user = user;
-    d_path = path;
+  Session::Session (const String& name, const String& info, const t_long mage) :
+    Taggable (name, info) {
+    d_skey = sess_new_skey ();
     d_ctim = Time::gettclk ();
     d_mtim = d_ctim;
-    d_etim = 0;
+    d_mage = (mage < 0LL) ? 0LL : mage;
+  }
+
+  // copy construct this session object
+
+  Session::Session (const Session& that) {
+    that.rdlock ();
+    try {
+      Taggable::operator = (that);
+      d_skey = that.d_skey;
+      d_shnm = that.d_shnm;
+      d_shid = that.d_shid;
+      d_path = that.d_path;
+      d_ctim = that.d_ctim;
+      d_mtim = that.d_mtim;
+      d_mage = that.d_mage;
+      that.unlock ();
+    } catch (...) {
+      that.unlock ();
+      throw;
+    }
+  }
+
+  // assign a session object to this one
+
+  Session& Session::operator = (const Session& that) {
+    // check for self-assignation
+    if (this == &that) return *this;
+    // lock and assign
+    wrlock ();
+    that.rdlock ();
+    try {
+      Taggable::operator = (that);
+      d_skey = that.d_skey;
+      d_shnm = that.d_shnm;
+      d_shid = that.d_shid;
+      d_path = that.d_path;
+      d_ctim = that.d_ctim;
+      d_mtim = that.d_mtim;
+      d_mage = that.d_mage;
+      unlock ();
+      that.unlock ();
+      return *this;
+    } catch (...) {
+      unlock ();
+      that.unlock ();
+      throw;
+    }
   }
 
   // return the class name
@@ -91,23 +142,37 @@ namespace afnix {
     return "Session";
   }
 
-  // return the session serial code
+  // return a clone of this object
 
-  t_byte Session::serialid (void) const {
-    return SERIAL_SESS_ID;
+  Object* Session::clone (void) const {
+    return new Session (*this);
+  }
+  
+  // return the serial did
+
+  t_word Session::getdid (void) const {
+    return SRL_DEOD_NWG;
   }
 
+  // return the serial sid
+
+  t_word Session::getsid (void) const {
+    return SRL_SESS_SID;
+  }
+  
   // serialize this session
 
   void Session::wrstream (OutputStream& os) const {
     rdlock ();
     try {
-      d_name.wrstream    (os);
-      d_user.wrstream    (os);
-      d_shid.wrstream    (os);
-      nwg_wrlong (d_ctim, os);
-      nwg_wrlong (d_mtim, os);
-      nwg_wrlong (d_etim, os);
+      Taggable::wrstream (os);
+      d_skey.wrstream (os);
+      d_shnm.wrstream (os);
+      d_shid.wrstream (os);
+      d_path.wrstream (os);
+      Serial::wrlong (d_ctim, os);
+      Serial::wrlong (d_mtim, os);
+      Serial::wrlong (d_mage, os);
       unlock ();
     } catch (...) {
       unlock ();
@@ -120,12 +185,14 @@ namespace afnix {
   void Session::rdstream (InputStream& is) {
     wrlock ();
     try {
-      d_name.rdstream     (is);
-      d_user.rdstream     (is);
+      Taggable::rdstream (is);
+      d_skey.rdstream     (is);
+      d_shnm.rdstream     (is);
       d_shid.rdstream     (is);
-      d_ctim = nwg_rdlong (is);
-      d_mtim = nwg_rdlong (is);
-      d_etim = nwg_rdlong (is);
+      d_path.rdstream     (is);
+      d_ctim = Serial::rdlong (is);
+      d_mtim = Serial::rdlong (is);
+      d_mage = Serial::rdlong (is);
       unlock ();
     } catch (...) {
       unlock ();
@@ -133,21 +200,7 @@ namespace afnix {
     }
   }
 
-  // get the session name
-
-  String Session::getname (void) const {
-    rdlock ();
-    try {
-      String result = d_name;
-      unlock ();
-      return result;
-    } catch (...) {
-      unlock ();
-      throw;
-    }
-  }
-
-  // set the session id
+  // set the session hash id
   
   void Session::setshid (const String& shid) {
     wrlock ();
@@ -161,40 +214,20 @@ namespace afnix {
     }
   }
 
-  // get the session id
+  // get the session hash id
 
   String Session::getshid (void) const {
     rdlock ();
     try {
+      // prepare result
       String result = d_shid;
-      unlock ();
-      return result;
-    } catch (...) {
-      unlock ();
-      throw;
-    }
-  }
-
-  // set the session user
-  
-  void Session::setuser (const String& user) {
-    wrlock ();
-    try {
-      d_user = user;
-      d_mtim = Time::gettclk ();
-      unlock ();
-    } catch (...) {
-      unlock ();
-      throw;
-    }
-  }
-
-  // get the session user
-
-  String Session::getuser (void) const {
-    rdlock ();
-    try {
-      String result = d_user;
+      // check for nil and rehash
+      if (result.isnil () == true) {
+	// prepare the hash value
+	String hs = d_name + d_skey;
+	// hash the string
+	result = Utility::hashs (hs);
+      }
       unlock ();
       return result;
     } catch (...) {
@@ -267,7 +300,7 @@ namespace afnix {
       // get the current time
       t_long time = Time::gettclk ();
       // check for expiration
-      bool result = (d_etim == 0) ? false : (d_etim <= time);
+      bool result = (d_mage == 0LL) ? false : (d_ctim + d_mage <= time);
       unlock ();
       return result;
     } catch (...) {
@@ -276,13 +309,14 @@ namespace afnix {
     }
   }
 
+
   // set the session expiration time
-  
-  void Session::setetim (const t_long time) {
+  void Session::setetim (const t_long etim) {
     wrlock ();
     try {
-      d_etim = time;
       d_mtim = Time::gettclk ();
+      d_mage = etim - d_ctim;
+      if (d_mage < 0LL) d_mage = 0LL;
       unlock ();
     } catch (...) {
       unlock ();
@@ -295,7 +329,7 @@ namespace afnix {
   t_long Session::getetim (void) const {
     rdlock ();
     try {
-      t_long result = d_etim;
+      t_long result = d_ctim + d_mage;
       unlock ();
       return result;
     } catch (...) {
@@ -303,14 +337,20 @@ namespace afnix {
       throw;
     }
   }
-  
-  // mark the session as expired by using the creation time
 
-  void Session::mrketim (void) {
-    wrlock ();
+  // get the session valid remaining time
+
+  t_long Session::getrtim (void) const {
+    rdlock ();
     try {
-      d_etim = d_ctim;
+      // get the current time
+      t_long time = Time::gettclk ();
+      // compute remaining time
+      t_long vldt = (d_mage == 0) ? 0 : (d_ctim + d_mage - time);
+      if (vldt < 0) vldt = 0;
+      // unlock and return
       unlock ();
+      return vldt;
     } catch (...) {
       unlock ();
       throw;
@@ -323,7 +363,8 @@ namespace afnix {
     wrlock ();
     try {
       d_mtim = Time::gettclk ();
-      d_etim = d_mtim + mage;
+      d_mage = mage;
+      if (d_mage < 0LL) d_mage = 0LL;
       unlock ();
     } catch (...) {
       unlock ();
@@ -331,54 +372,84 @@ namespace afnix {
     }
   }
 
-  // get the session valid remaining time
+  // get the session maximum age
 
-  t_long Session::getvldt (void) const {
+  t_long Session::getmage (void) const {
     rdlock ();
     try {
-      // get the current time
-      t_long time = Time::gettclk ();
-      // compute remaining time
-      t_long vldt = (d_etim == 0) ? 0 : (d_etim - time);
-      if (vldt < 0) vldt = 0;
-      // unlock and return
+      t_long result = d_mage;
       unlock ();
-      return vldt;
+      return result;
     } catch (...) {
       unlock ();
       throw;
     }
   }
 
+  // bake a new cookie from the session information
+
+  Cookie* Session::getcookie (const String& name) {
+    rdlock ();
+    try {
+      // save the session hash name
+      d_shnm = name;
+      d_mtim = Time::gettclk ();
+      // create a new cookie
+      Cookie* result = new Cookie (d_shnm, getshid (), d_mage);
+      unlock ();
+      return result;
+    } catch (...) {
+      unlock ();
+      throw;
+    }
+  }
+
+  // close a session by reseting the maximum age
+
+  Cookie* Session::close (void) {
+    wrlock ();
+    try {
+      // close the session
+      setmage (0LL);
+      // get a closing cookie
+      Cookie* result = new Cookie (d_shnm, getshid (), d_mage);
+      // done
+      unlock ();
+      return result;
+    } catch (...) {
+      unlock ();
+      throw;
+    }
+  }
+      
   // -------------------------------------------------------------------------
   // - object section                                                        -
   // -------------------------------------------------------------------------
 
   // the quark zone
-  static const long QUARK_ZONE_LENGTH = 15;
+  static const long QUARK_ZONE_LENGTH = 16;
   static QuarkZone  zone (QUARK_ZONE_LENGTH);
 
   // the object supported quarks
+  static const long QUARK_CLOSE   = zone.intern ("close");
   static const long QUARK_ETIMP   = zone.intern ("expire-p");
-  static const long QUARK_GETNAME = zone.intern ("get-name");
   static const long QUARK_SETSHID = zone.intern ("set-hash-id");
   static const long QUARK_GETSHID = zone.intern ("get-hash-id");
-  static const long QUARK_GETUSER = zone.intern ("get-user");
-  static const long QUARK_SETUSER = zone.intern ("set-user");
   static const long QUARK_GETPATH = zone.intern ("get-path");
   static const long QUARK_SETPATH = zone.intern ("set-path");
-  static const long QUARK_SETMAGE = zone.intern ("set-maximum-age");
-  static const long QUARK_MRKETIM = zone.intern ("mark-expired");
-  static const long QUARK_GETVLDT = zone.intern ("get-valid-time");
+  static const long QUARK_GETCOOK = zone.intern ("get-cookie");
+  static const long QUARK_SETMAGE = zone.intern ("set-max-age");
+  static const long QUARK_GETMAGE = zone.intern ("get-max-age");
   static const long QUARK_SETETIM = zone.intern ("set-expire-time");
   static const long QUARK_GETETIM = zone.intern ("get-expire-time");
   static const long QUARK_GETCTIM = zone.intern ("get-creation-time");
+  static const long QUARK_GETRTIM = zone.intern ("get-remaining-time");
   static const long QUARK_GETMTIM = zone.intern ("get-modification-time");
 
   // create a new object in a generic way
 
   Object* Session::mknew (Vector* argv) {
-    long argc = (argv == nilp) ? 0 : argv->length ();
+    long argc = (argv == nullptr) ? 0 : argv->length ();
 
     // check for 1 argument
     if (argc == 1) {
@@ -388,15 +459,15 @@ namespace afnix {
     // check for 2 arguments
     if (argc == 2) {
       String name = argv->getstring (0);
-      String user = argv->getstring (1);
-      return new Session (name, user);
+      String info = argv->getstring (1);
+      return new Session (name, info);
     }
     // check for 3 arguments
     if (argc == 3) {
       String name = argv->getstring (0);
-      String user = argv->getstring (1);
-      String path = argv->getstring (2);
-      return new Session (name, user, path);
+      String info = argv->getstring (1);
+      t_long mage = argv->getlong   (2);
+      return new Session (name, info, mage);
     }
     // invalid arguments
     throw Exception ("argument-error", "invalid arguments with session");
@@ -406,40 +477,38 @@ namespace afnix {
 
   bool Session::isquark (const long quark, const bool hflg) const {
     rdlock ();
-    if (zone.exists (quark) == true) {
+    try {
+      if (zone.exists (quark) == true) {
+	unlock ();
+	return true;
+      }
+      bool result = hflg ? Taggable::isquark (quark, hflg) : false;
       unlock ();
-      return true;
+      return result;
+    } catch (...) {
+      unlock ();
+      throw;
     }
-    bool result = hflg ? Nameable::isquark (quark, hflg) : false;
-    if (result == false) {
-      result = hflg ? Serial::isquark (quark, hflg) : false;
-    }
-    unlock ();
-    return result;
   }
 
   // apply this object with a set of arguments and a quark
 
-  Object* Session::apply (Runnable* robj, Nameset* nset, const long quark,
-			 Vector* argv) {
+  Object* Session::apply (Evaluable* zobj, Nameset* nset, const long quark,
+			  Vector* argv) {
     // get the number of arguments
-    long argc = (argv == nilp) ? 0 : argv->length ();
+    long argc = (argv == nullptr) ? 0 : argv->length ();
     
     // dispatch 0 argument
     if (argc == 0) {
       if (quark == QUARK_ETIMP)   return new Boolean (isetim  ());
-      if (quark == QUARK_GETNAME) return new String  (getname ());
       if (quark == QUARK_GETSHID) return new String  (getshid ());
-      if (quark == QUARK_GETUSER) return new String  (getuser ());
       if (quark == QUARK_GETPATH) return new String  (getpath ());
-      if (quark == QUARK_GETVLDT) return new Integer (getvldt ());
       if (quark == QUARK_GETETIM) return new Integer (getetim ());
       if (quark == QUARK_GETCTIM) return new Integer (getctim ());
       if (quark == QUARK_GETMTIM) return new Integer (getmtim ());
-      if (quark == QUARK_MRKETIM) {
-	mrketim ();
-	return nilp;
-      }
+      if (quark == QUARK_GETMAGE) return new Integer (getmage ());
+      if (quark == QUARK_GETRTIM) return new Integer (getrtim ());
+      if (quark == QUARK_CLOSE)   return close ();
     }
 
     // dispatch 1 argument
@@ -447,34 +516,29 @@ namespace afnix {
       if (quark == QUARK_SETSHID) {
 	String shid = argv->getstring (0);
 	setshid (shid);
-	return nilp;
-      }   
-      if (quark == QUARK_SETUSER) {
-	String user = argv->getstring (0);
-	setuser (user);
-	return nilp;
-      }   
+	return nullptr;
+      }
       if (quark == QUARK_SETPATH) {
 	String path = argv->getstring (0);
 	setpath (path);
-	return nilp;
+	return nullptr;
       }
       if (quark == QUARK_SETETIM) {
 	t_long time = argv->getlong (0);
 	setetim (time);
-	return nilp;
+	return nullptr;
       }
       if (quark == QUARK_SETMAGE) {
 	t_long mage = argv->getlong (0);
 	setmage (mage);
-	return nilp;
+	return nullptr;
+      }
+      if (quark == QUARK_GETCOOK) {
+	String name = argv->getstring (0);
+	return getcookie (name);
       }
     }
-    // check the nameable method
-    if (Nameable::isquark (quark, true) == true) {
-      return Nameable::apply (robj, nset, quark, argv);
-    }
-    // call the serial method
-    return Serial::apply (robj, nset, quark, argv);
+    // call the taggable method
+    return Taggable::apply (zobj, nset, quark, argv);
   }
 }
