@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "Real.hpp"
@@ -35,12 +35,12 @@ namespace afnix {
   static const String PN_PHY_U0 = "µ₀";
   static const String PI_PHY_U0 = "MAGNETIC PERMEABILITY IN VACUUM";
   static const t_real PV_PHY_U0 = 1.25663706212E-6;
-  static const String PU_PHY_U0 = "H/m";
+  static const String PU_PHY_U0 = "H.m^-1";
 
   static const String PN_PHY_E0 = "ε₀";
   static const String PI_PHY_E0 = "ELECTRIC PERMITIVITY IN VACUUM";
   static const t_real PV_PHY_E0 = 8.8541878128E-12;
-  static const String PU_PHY_E0 = "F/m";
+  static const String PU_PHY_E0 = "F.m^-1";
 
   static const String PN_PHY_G  = "G";
   static const String PI_PHY_G  = "CONSTANT OF GRAVITATION";
@@ -50,12 +50,12 @@ namespace afnix {
   static const String PN_PHY_H  = "h";
   static const String PI_PHY_H  = "PLANCK CONSTANT";
   static const t_real PV_PHY_H  = 6.62607015E-34;
-  static const String PU_PHY_H  = "J⋅s";
+  static const String PU_PHY_H  = "J.s";
 
   static const String PN_PHY_HB = "ħ";
   static const String PI_PHY_HB = "REDUCED PLANCK CONSTANT";
   static const t_real PV_PHY_HB = 1.054571817E-34;
-  static const String PU_PHY_HB = "J⋅s";
+  static const String PU_PHY_HB = "J.s";
 
   static const String PN_PHY_E  = "e";
   static const String PI_PHY_E  = "ELEMENTARY CHARGE";
