@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2022 amaury darsch                                   -
+// - copyright (c) 1999-2023 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "Unit.hpp"
@@ -87,7 +87,7 @@ namespace afnix {
     desc.d_expn = 1L;
   }
 
-  // copy a dscriptor array
+  // copy a descriptor array
   static s_unit* si_cpy_desc (const long size, const s_unit* desc) {
     // check for null
     if ((size == 0L) || (desc == nullptr)) return nullptr;
@@ -286,6 +286,14 @@ namespace afnix {
     }
     return result;
   }
+
+  // normalize a descriptor
+  static s_unit si_normalize (const s_unit& desc) {
+    s_unit result = desc;
+    result.d_pfix = s_unit::PFIX_PONE;
+    if (result.d_base == s_unit::BASE_M) result.d_pfix = s_unit::PFIX_KILO;
+    return result;
+  }
   
   // map a descriptor to a string
   static String si_tostring (const s_unit& desc) {
@@ -712,6 +720,98 @@ namespace afnix {
       throw;
     }
   }
+
+  // normalize a unit
+
+  Unit Unit::normalize (void) const {
+    rdlock ();
+    try {
+      // copy construct the result
+      Unit result = *this;
+      // get the scaling factor
+      result.d_scal = toscaled ();
+      // normalize the descriptors
+      for (long k = 0L; k < d_size; k++) {
+	result.p_desc[k] = si_normalize(p_desc[k]);
+      }
+      unlock ();
+      return result;
+    } catch (...) {
+      unlock ();
+      throw;
+    }
+  }
+
+  // rescale this unit by another one
+
+  t_real Unit::rescale (const Unit& unit) const {
+    rdlock ();
+    try {
+      // normalize all units
+      Unit sunt = normalize ();
+      Unit runt = unit.normalize ();
+      // check for homogeneous units
+      if (sunt.tosi () != runt.tosi ()) {
+	throw Exception ("unit-error", "non homogeneous units to rescale");
+      }
+      // prepare scled result
+      t_real result = toscaled() / unit.toscaled ();
+      unlock ();
+      return result;      
+    } catch (...) {
+      unlock ();
+      throw;
+    }
+  }
+
+  // rescale this unit by unit name
+
+  t_real Unit::rescale (const String& unam) const {
+    rdlock ();
+    try {
+      // create a unit by string
+      Unit unit = Unit::create (unam);
+      // rescale the unit
+      t_real result = rescale (unit);
+      unlock ();
+      return result;
+    } catch (...) {
+      unlock ();
+      throw;
+    }
+  }
+  
+  // rescale a scalar with this unit and another one
+
+  t_real Unit::rescale (const Unit& unit, const t_real uval) const {
+    rdlock ();
+    try {
+      // rescale the unit
+      t_real f = rescale (unit);
+      t_real result = uval * f;
+      unlock ();
+      return result;
+    } catch (...) {
+      unlock ();
+      throw;
+    }
+  }
+  
+  // rescale a scalar with this unit and another one
+
+  t_real Unit::rescale (const String& unam, const t_real uval) const {
+    rdlock ();
+    try {
+      // rescale the unit
+      t_real f = rescale (unam);
+      t_real result = uval * f;
+      unlock ();
+      return result;
+    } catch (...) {
+      unlock ();
+      throw;
+    }
+  }
   
   // parse a unit name
 
@@ -760,25 +860,27 @@ namespace afnix {
   t_real operator * (const t_real uval, const Unit& unit) {
     return uval * unit.getscal() * unit.tofactor();
   }
-  
+
   // -------------------------------------------------------------------------
   // - object section                                                        -
   // -------------------------------------------------------------------------
 
   
   // the quark zone
-  static const long QUARK_ZONE_LENGTH = 7;
+  static const long QUARK_ZONE_LENGTH = 10;
   static QuarkZone  zone (QUARK_ZONE_LENGTH);
 
   // the object supported quarks
-  static const long QUARK_MUL     = zone.intern ("mul");
-  static const long QUARK_TOSI    = zone.intern ("to-si");
-  static const long QUARK_RESET   = zone.intern ("reset");
-  static const long QUARK_PARSE   = zone.intern ("parse");
-  static const long QUARK_TOSTNG  = zone.intern ("to-string");
-  static const long QUARK_TOFACT  = zone.intern ("to-factor");
-  static const long QUARK_TOSCLF  = zone.intern ("to-scaled-factor");
-  static const long QUARK_GETSCAL = zone.intern ("get-scaling-factor");
+  static const long QUARK_MUL       = zone.intern ("mul");
+  static const long QUARK_TOSI      = zone.intern ("to-si");
+  static const long QUARK_RESET     = zone.intern ("reset");
+  static const long QUARK_PARSE     = zone.intern ("parse");
+  static const long QUARK_TOSTNG    = zone.intern ("to-string");
+  static const long QUARK_TOFACT    = zone.intern ("to-factor");
+  static const long QUARK_TOSCLF    = zone.intern ("to-scaled-factor");
+  static const long QUARK_GETSCAL   = zone.intern ("get-scaling-factor");
+  static const long QUARK_RESCALE   = zone.intern ("rescale");
+  static const long QUARK_NORMALIZE = zone.intern ("normalize");
 
   // create a new object in a generic way
 
@@ -836,11 +938,12 @@ namespace afnix {
 
     // dispatch 0 argument
     if (argc == 0) {
-      if (quark == QUARK_TOSI)    return new String (tosi     ());
-      if (quark == QUARK_TOFACT)  return new Real   (tofactor ());
-      if (quark == QUARK_TOSCLF)  return new Real   (toscaled ());
-      if (quark == QUARK_TOSTNG)  return new String (tostring ());
-      if (quark == QUARK_GETSCAL) return new Real   (getscal  ());
+      if (quark == QUARK_TOSI)      return new String (tosi      ());
+      if (quark == QUARK_TOFACT)    return new Real   (tofactor  ());
+      if (quark == QUARK_TOSCLF)    return new Real   (toscaled  ());
+      if (quark == QUARK_TOSTNG)    return new String (tostring  ());
+      if (quark == QUARK_GETSCAL)   return new Real   (getscal   ());
+      if (quark == QUARK_NORMALIZE) return new Unit   (normalize ());
       if (quark == QUARK_RESET) {
 	reset ();
 	return nullptr;
@@ -856,6 +959,18 @@ namespace afnix {
 	String dval = argv->getstring (0);
 	return new Boolean (parse (dval));
       }
+      if (quark == QUARK_RESCALE) {
+	Object* obj = argv->get (0);
+	// check for a unit
+	auto unit = dynamic_cast<Unit*>(obj);
+	if (unit != nullptr) return new Real (rescale (*unit));
+	// check for a string
+	auto unam = dynamic_cast<String*>(obj);
+	if (unam != nullptr) return new Real (rescale (*unam));
+	// invalid object
+	throw Exception ("type-error", "invalid object as unit",
+			 Object::repr (obj));
+      }
     }
     // dispatch 2 arguments
     if (argc == 2) {
@@ -864,6 +979,19 @@ namespace afnix {
 	long   cnst = argv->getlong (1);
 	return new Boolean (parse (dval, cnst));
       }
+      if (quark == QUARK_RESCALE) {
+	Object* obj = argv->get (0);
+	t_real uval = argv->getreal (1);
+	// check for a unit
+	auto unit = dynamic_cast<Unit*>(obj);
+	if (unit != nullptr) return new Real (rescale (*unit, uval));
+	// check for a string
+	auto unam = dynamic_cast<String*>(obj);
+	if (unam != nullptr) return new Real (rescale (*unam, uval));
+	// invalid object
+	throw Exception ("type-error", "invalid object as unit",
+			 Object::repr (obj));
+      }
     }
     // call the infoable method
     return Infoable::apply (zobj, nset, quark, argv);
